/* -*- mode:c -*- */
/** @file
 ** @brief Maestro Pool Client implementation
 **/
/*
 * Copyright (C) 2020 Cray Computer GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MAESTRO_I_POOL_CLIENT_H_
#define MAESTRO_I_POOL_CLIENT_H_ 1

/**@addtogroup MSTRO_Internal
 **@{
 **/

/**@defgroup MSTRO_I_POOL_CLIENT Maestro Pool Client implementation
 **@{
 ** This is the Pool Client handling interface
 **/


#include "maestro/status.h"
#include "maestro/i_pool_manager_protocol.h"
#include "protocols/mstro_pool.pb-c.h"

#include <stdbool.h>
#include <stdatomic.h>


/* Pool client "Handle incoming Message" dispatcher function
 *
 * If the message handling needs to be voluntarily preempted, the
 * handler returns MSTRO_WOULDBKLOCK and fills in @arg
 * *restart_closure, expecting to be called again with the same
 * envelope and the same restart_closure to retry processing.
 */
mstro_status
mstro_pc_handle_msg(const struct mstro_msg_envelope *envelope,
                    void ** restart_closure);

/**@} (end of group MSTRO_I_POOL_CLIENT) */
/**@} (end of group MSTRO_Internal) */

#endif /* MAESTRO_I_POOL_CLIENT_H_ */
