%module maestro_core
%include "typemaps.i"
%include "stdint.i"
%include "cpointer.i"
%include "exception.i"

/* No identifier conversion here. PEP-8 says
 *
 * > Function names should be lowercase, with words separated by underscores as
 * > necessary to improve readability.
 *
 * so our C API will be just fine.
 */

/***** typemaps *****/
/* mstro_status codes: suppress unless an error needs to be indicated. In that case: throw exeption */
%typemap(out) mstro_status  {
  $result = SWIG_Py_Void();
  if(MSTRO_OK!=$1) {
    SWIG_exception(SWIG_RuntimeError, mstro_status_description($1));
  }
}

/* swig needs help for opaque data types to generate a proper descriptor for the cell around them */
//typedef void* mstro_cdo;

/* CDO return values: */
%typemap(in, numinputs=0) mstro_cdo *result (mstro_cdo temp) {
  $1 = &temp;
}

%typemap(argout) mstro_cdo *result {
  /* Create shadow object (do not use SWIG_POINTER_NEW) */
  PyObject * temp = SWIG_NewPointerObj(*$1,
                                       $descriptor(mstro_cdo),
                                       SWIG_POINTER_NOSHADOW);
  /* if multiple results: ensure result is a list */
  if(VOID_Object==$result) {
    /* we're producing the first result */
    $result = temp;
  } else {
    /* coerce previous result to list and append */
    if (!PyList_Check($result)) {
      PyObject * tt = $result;
      $result = PyList_New(1);
      PyList_SetItem($result, 0, tt);
    }
    PyList_Append($result, temp);
    Py_DECREF(temp);
  }
}

/* char ** result args */
%typemap(in, numinputs=0) char **result_p (char *tmp) {
  $1 = &tmp;
}
%typemap(argout) char **result_p (PyObject* obj) {
 /* wrap char** putput to python unicode object */
 obj = PyUnicode_FromString(*$1);
 $result = SWIG_Python_AppendOutput($result,obj);
}
/* unicode object is a copy and owned by python, so temporary object returned by maestro can be freed */
%typemap(freearg) char** result_p {
    free(*$1);
}

%{
#define SWIG_FILE_WITH_INIT
  #include "maestro.h"
  #include "maestro/attributes.h"
  #include "maestro/cdo.h"
  #include "maestro/config.h"
  #include "maestro/core.h"
  #include "maestro/env.h"
  #include "maestro/groups.h"
  #include "maestro/logging.h"
  #include "maestro/msm.h"
  #include "maestro/pool.h"
  #include "maestro/pool_manager.h"
  #include "maestro/scope.h"
  #include "maestro/status.h"
%}


%include "maestro.h"
%include "maestro/attributes.h"
%include "maestro/cdo.h"
%include "maestro/config.h"
%include "maestro/core.h"
%include "maestro/env.h"
%include "maestro/groups.h"
// %include "maestro/logging.h"
%include "maestro/msm.h"
%include "maestro/pool.h"
%include "maestro/pool_manager.h"
%include "maestro/scope.h"
%include "maestro/status.h"
